<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0"
			xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
			xmlns:saxon="http://saxon.sf.net/"
			xmlns:file="java:java.io.File">
			
	<xsl:output method="html" indent="yes"/>
	<xsl:strip-space elements="*"/>
	
	<xsl:variable name="units-time" select="('ms', 's', 'min', 'h')"/>
	<xsl:variable name="units-date" select="('d', 'w', 'mon', 'y')"/>	

	<xsl:variable name="units-length" select="('mm', 'cm', 'm', 'km', 'in', 'ft', 'yd', 'mi')"/>
	<xsl:variable name="units-temperature" select="('C', 'F')"/>
	<xsl:variable name="units-area" select="('qm', 'a', 'ha')"/>
	<xsl:variable name="units-volume" select="('ml', 'l', 'hl', 'floz', 'gal')"/>
	<xsl:variable name="units-mass" select="('mg', 'g', 'kg', 'to')"/>
	<xsl:variable name="units-energy" select="('J', 'kcal', 'kWh')"/>
	<xsl:variable name="units-pressure" select="('Pa', 'mbar', 'bar', 'mmHg', 'mmH2O')"/>	
	<xsl:variable name="units-currency" select="('Euro', 'cent', 'USD')"/>
	<xsl:variable name="units-parts" select="('ppm', 'ppb', '%')"/>	
	
	<!-- match root node -->
	<xsl:template match="/">
		<xsl:variable name="file-name" select="file:getName(file:new(string(base-uri())))"/>
		<html>
			<head>
				<title>Asbru Semantics Check of <xsl:value-of select="$file-name"/></title>
				<style type="text/css">
					<xsl:comment>
						a { text-decoration:none; font-weight:normal; white-space:nowrap; }
						ul { margin-top:-1; margin-bottom:-1;}
					</xsl:comment>
				</style>
			</head>
			<body>
				<h1>Consistency Check of Asbru file <code><xsl:value-of select="$file-name"/></code></h1>
<!-- 				<ul>
					<li><a href="#Plans">Plans</a></li>
					<li><a href="#Parameters">Parameters</a></li>
					<li><a href="#Variables">Variables</a></li>
					<li><a href="#Arguments">Arguments</a></li>
					<li><a href="#Return-Values">Return-Values</a></li>
					<li><a href="#Constants">Constants</a></li>
					<li><a href="#Types">Types</a></li>
					<li><a href="#Units">Units</a></li>
					<li><a href="#Contexts">Contexts</a></li>
					<li><a href="#Misc">Misc. Stuff</a></li>
				</ul> -->
				<b>Content: </b><a href="#Plans">Plans</a>, <a href="#Parameters">Parameters</a>, <a href="#Variables">Variables</a>, <a href="#Arguments">Arguments</a>, <a href="#Return-Values">Return-Values</a>, <a href="#Constants">Constants</a>, <a href="#Types">Types</a>, <a href="#Units">Units</a>, <a href="#Contexts">Contexts</a>, <a href="#Misc">Misc. Stuff</a>
				<br/><hr/><br/>
				<a name="Plans"><b><big>Plans: </big></b></a>
				<ul>
					<li>Undefined Plans<!--  (<code>plan-schema, static-plan-pointer, refers-to</code> without <code>plan</code>) -->
						<xsl:variable name="plan-names" select="//plan/@name"/>
						<xsl:variable name="undefined-plans" select="//plan-schema[not(@name=$plan-names)]|(//static-plan-pointer|//refers-to|//derived-from)[not(@plan-name=$plan-names)]"/>
						<xsl:choose>
							<xsl:when test="$undefined-plans"> (<font color="red"><b><xsl:value-of select="count($undefined-plans)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$undefined-plans" mode="undefined">
									<!-- <xsl:sort select="@name|@plan-name"/> -->
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Uncalled Plans<small> - if 0 or more than one</small><!--  (<code>plan</code> without any matching <code>plan-schema</code>'s) -->
						<xsl:variable name="uncalled-plans" select="//plan[not(@name=//plan-schema/@name)]"/>
						<xsl:choose>
							<xsl:when test="count($uncalled-plans) != 1"> (<font color="red"><b><xsl:value-of select="count($uncalled-plans)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$uncalled-plans" mode="unused">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>one</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Duplicate Plans
						<xsl:variable name="duplicate-plans" select="for $p in //plan return if (count(//plan[@name=$p/@name])&gt;1) then $p else ()"/>
						<xsl:choose>
							<xsl:when test="$duplicate-plans"> (<font color="red"><b><xsl:value-of select="count(distinct-values($duplicate-plans/@name))"/></b></font>):
								<ul>
									<xsl:for-each-group select="$duplicate-plans" group-by="@name">
										<xsl:sort select="@name"/>
										<li>Parameter "<i><xsl:value-of select="@name"/></i>" (<font color="red"><b><xsl:value-of select="count(current-group())"/></b></font>):<br/>
											<xsl:for-each select="current-group()"><a href="{saxon:path()}"><code><xsl:value-of select="node-name()"/></code> #<xsl:value-of select="position()"/></a><xsl:if test="position()&lt;last()">, </xsl:if>
											</xsl:for-each>
										</li>
									</xsl:for-each-group>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>					
				</ul>
				<hr/>
				
				<a name="Parameters"><b><big>Parameters: </big></b></a>
				<ul>
					<li>Undefined Parameters<!--  (<code>paramter-ref</code>'s, <code>paramter-proposition</code>'s or <code>one-of</code>'s without matching <code>paramter-def</code>) -->
						<xsl:variable name="parameter-names" select="//parameter-def/@name"/>
						<xsl:variable name="undefined-parameters" select="(//parameter-ref|//one-of)[not(@name=$parameter-names)]|(//parameter-proposition|//is-manual|//is-automatic)[not(@parameter-name=$parameter-names)]|(//plans//parameter-assignment|//plan-effect)[not(@parameter=$parameter-names)]"/>
						<xsl:choose>
							<xsl:when test="$undefined-parameters"> (<font color="red"><b><xsl:value-of select="count($undefined-parameters)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$undefined-parameters" mode="undefined">
									<!-- <xsl:sort select="@name|@parameter-name|@parameter"/> -->
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Circular References in domain part
					<xsl:variable name="circular-parameters" select="for $var in //domain//parameter-ref return if ($var/@name=$var/ancestor::parameter-def/@name) then $var else ()"/>
						<xsl:choose>
							<xsl:when test="$circular-parameters"> (<font color="red"><b><xsl:value-of select="count($circular-parameters)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$circular-parameters" mode="unused">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><font color="green"><b>none</b></font></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Unused Parameters<small> - no parameter-ref's or parameter-proposition's</small><!-- <code>parameter-def</code> without any matching <code>paramter-ref</code>'s, <code>paramter-proposition</code>'s or <code>one-of</code>'s -->
						<xsl:variable name="unused-parameters" select="//parameter-def[not(@name=((//parameter-ref|//one-of)/@name|//parameter-proposition/@parameter-name))]"/>
						<xsl:choose>
							<xsl:when test="$unused-parameters"> (<font color="red"><b><xsl:value-of select="count($unused-parameters)"/></b></font>):
								<ul>
								<!-- <xsl:apply-templates select="$unused-parameters" mode="unused">
									<xsl:sort select="@name"/>
								</xsl:apply-templates> -->
								<xsl:for-each select="$unused-parameters">
									<li><a href="{saxon:path()}"><xsl:value-of select="@name"/></a>
										<xsl:if test="//plans//parameter-assignment[@parameter=current()/@name]"> (parameter-assignment)</xsl:if>
									</li>
								</xsl:for-each>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Duplicate Parameters
						<xsl:variable name="duplicate-parameters" select="for $p in //parameter-def return if (count(//parameter-def[@name=$p/@name])&gt;1) then $p else ()"/>
						<xsl:choose>
							<xsl:when test="$duplicate-parameters"> (<font color="red"><b><xsl:value-of select="count(distinct-values($duplicate-parameters/@name))"/></b></font>):
								<ul>
									<xsl:for-each-group select="$duplicate-parameters" group-by="@name">
										<xsl:sort select="@name"/>
										<li>Parameter "<i><xsl:value-of select="@name"/></i>" (<font color="red"><b><xsl:value-of select="count(current-group())"/></b></font>):<br/>
											<xsl:for-each select="current-group()"><a href="{saxon:path()}"><code><xsl:value-of select="node-name()"/></code> #<xsl:value-of select="position()"/></a><xsl:if test="position()&lt;last()">, </xsl:if>
											</xsl:for-each>
										</li>
									</xsl:for-each-group>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
				</ul>
				<hr/>
				
				<a name="Variables"><b><big>Variables:</big></b></a>
				<ul>
					<li>Undefined Variables<!-- <code>variable-ref</code> without <code>variable-def</code> -->
						<xsl:variable name="global-variables" select="/plan-library/(domain-defs/domain|/library-defs)/variable-def/@name"/>
						<xsl:variable name="undefined-variables" select="(for $var in (//variable-ref|//is-known-variable|//is-unknown-variable|//return-value-assignment) return if (not( ($var/@name|$var/@variable-name) = (($var/ancestor::plan/value-defs/variable-def/@name)|$global-variables))) then $var else ())"/>
						<xsl:choose>
							<xsl:when test="$undefined-variables"> (<font color="red"><b><xsl:value-of select="count($undefined-variables)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$undefined-variables" mode="undefined">
									<!-- <xsl:sort select="@name|@variable-name"/> -->
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Referenced, but never assigned variables
						<xsl:variable name="unused-variables" select="(//domain|//library-defs)/variable-def[not(@name=(//variable-assignment/@variable|//return-value-assignment/@variable-name)|scalar-def/initial-value)][@name=(//variable-ref|//is-known-variable|//is-unknown-variable)/@name]"/>
						<xsl:choose>
							<xsl:when test="$unused-variables"> (<font color="red"><b><xsl:value-of select="count($unused-variables)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$unused-variables" mode="unused">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Assigned, but never referenced variables
						<xsl:variable name="unused-variables" select="(//domain|//library-defs)/variable-def[@name=(//variable-assignment/@variable|//return-value-assignment/@variable-name)|scalar-def/initial-value][not(@name=(//variable-ref|//is-known-variable|//is-unknown-variable)/@name)]"/>
						<xsl:choose>
							<xsl:when test="$unused-variables"> (<font color="red"><b><xsl:value-of select="count($unused-variables)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$unused-variables" mode="unused">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>					
					<li>Unused Variables<small> - defined, but never referenced or assigned</small><!--  (<code>variable-def</code> without any <code>variable-ref</code>'s) -->
						<xsl:variable name="unused-variables" select="(//domain|//library-defs)/variable-def[not(@name=((//variable-ref|//is-known-variable|//is-unknown-variable)/@name|//variable-assignment/@variable|//return-value-assignment/@variable-name))] | //value-defs/variable-def[not(@name=((ancestor::plan//(variable-ref|is-known-variable|is-unknown-variable)/@name)|ancestor::plan//variable-assignment/@variable|ancestor::plan//return-value-assignment/@variable-name))]"/>
						<xsl:choose>
							<xsl:when test="$unused-variables"> (<font color="red"><b><xsl:value-of select="count($unused-variables)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$unused-variables" mode="unused">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Duplicate Variables
						<xsl:variable name="duplicate-variables" select="for $v in //variable-def return if (count($v/ancestor::*/variable-def[@name=$v/@name])&gt;1) then $v else ()"/>
						<xsl:choose>
							<xsl:when test="$duplicate-variables"> (<font color="red"><b><xsl:value-of select="count(distinct-values($duplicate-variables/@name))"/></b></font>):
								<ul>
									<xsl:for-each-group select="$duplicate-variables" group-by="@name">
										<xsl:sort select="@name"/>
										<li>
											<xsl:variable name="local" select="current()/ancestor::value-defs"/>
											<xsl:choose>
												<xsl:when test="$local">local </xsl:when>
												<xsl:otherwise>global </xsl:otherwise>
											</xsl:choose>Variable "<i><xsl:value-of select="@name"/></i>"
											<xsl:if test="$local"> defined in Plan <a><xsl:attribute name="href"><xsl:apply-templates select="ancestor::plan" mode="xpath"/></xsl:attribute>"<i><xsl:value-of select="ancestor::plan/@name"/></i>"</a></xsl:if>
											(<font color="red"><b><xsl:value-of select="count(current-group())"/></b></font>):<br/>
											<xsl:for-each select="current-group()"><a href="{saxon:path()}"><code><xsl:value-of select="node-name()"/></code> #<xsl:value-of select="position()"/></a><xsl:if test="position()&lt;last()">, </xsl:if>
											</xsl:for-each>
										</li>
									</xsl:for-each-group>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					</ul>
					<hr/>
				
					<a name="Arguments"><b><big>Arguments:</big></b></a>
					<ul>
					<li>Undefined Arguments<!--  (<code>argument-ref</code> without <code>argument</code>) -->
						<xsl:variable name="undefined-arguments" select="//argument-ref[not(@name=ancestor::plan/arguments/argument/@name)]|(for $arg in //argument-value return if (not($arg/@name=//plan[@name=($arg/parent::plan-schema/@name)]/arguments/argument/@name)) then $arg else ())"/>
						<xsl:choose>
							<xsl:when test="$undefined-arguments"> (<font color="red"><b><xsl:value-of select="count($undefined-arguments)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$undefined-arguments" mode="undefined">
									<!-- <xsl:sort select="ancestor::plan/@name"/>
									<xsl:sort select="@name"/> -->
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Unused Arguments<!--  (<code>argument</code> without any <code>argument-ref</code>'s) -->
						<xsl:variable name="unused-arguments" select="//argument[not(@name=ancestor::plan//argument-ref/@name)]"/>
						<xsl:choose>
							<xsl:when test="$unused-arguments"> (<font color="red"><b><xsl:value-of select="count($unused-arguments)"/></b></font>):
								<ul>
								<xsl:for-each select="$unused-arguments">
									<xsl:sort select="@name"/>
									<li><i>argument</i> "<a href="{saxon:path()}"><xsl:value-of select="@name"/></a>" of <xsl:call-template name="info"/></li>
								</xsl:for-each>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Duplicate Arguments
						<xsl:variable name="duplicate-arguments" select="for $a in //argument return if (count($a/ancestor::*/argument[@name=$a/@name])&gt;1) then $a else ()"/>
						<xsl:choose>
							<xsl:when test="$duplicate-arguments"> (<font color="red"><b><xsl:value-of select="count(distinct-values($duplicate-arguments/@name))"/></b></font>):
								<ul>
									<xsl:for-each-group select="$duplicate-arguments" group-by="@name">
										<xsl:sort select="@name"/>
										<li>Argument "<i><xsl:value-of select="@name"/></i>" of Plan <a><xsl:attribute name="href"><xsl:apply-templates select="ancestor::plan" mode="xpath"/></xsl:attribute>"<i><xsl:value-of select="ancestor::plan/@name"/></i>"</a> (<font color="red"><b><xsl:value-of select="count(current-group())"/></b></font>):<br/>
											<xsl:for-each select="current-group()"><a href="{saxon:path()}"><code><xsl:value-of select="node-name()"/></code> #<xsl:value-of select="position()"/></a><xsl:if test="position()&lt;last()">, </xsl:if>
											</xsl:for-each>
										</li>
									</xsl:for-each-group>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					</ul>
					<hr/>
				
					<a name="Return-Values"><b><big>Return-Values:</big></b></a>
					<ul>
					<li>Undefined Return-Values<!--  (<code>return-value-assignment</code> without <code>return-value</code>) -->
						<xsl:variable name="undefined-return-values" select="(for $ret in //return-value-assignment return if (not($ret/@return-value-name=(//plan[@name=$ret/parent::plan-schema/@name]/returns/return-value/@name))) then $ret else ())"/>
						<xsl:choose>
							<xsl:when test="$undefined-return-values"> (<font color="red"><b><xsl:value-of select="count($undefined-return-values)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$undefined-return-values" mode="undefined">
									<!-- <xsl:sort select="@name"/> -->
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose> 
					</li>
					<li>Duplicate Return-Values
						<xsl:variable name="duplicate-return-values" select="for $d in //return-value return if (count($d/ancestor::*/return-value[@name=$d/@name])&gt;1) then $d else ()"/>
						<xsl:choose>
							<xsl:when test="$duplicate-return-values"> (<font color="red"><b><xsl:value-of select="count(distinct-values($duplicate-return-values/@name))"/></b></font>):
								<ul>
									<xsl:for-each-group select="$duplicate-return-values" group-by="@name">
										<xsl:sort select="@name"/>
										<li>Return-Value "<i><xsl:value-of select="@name"/></i>" of Plan <a><xsl:attribute name="href"><xsl:apply-templates select="ancestor::plan" mode="xpath"/></xsl:attribute>"<i><xsl:value-of select="ancestor::plan/@name"/></i>"</a> (<font color="red"><b><xsl:value-of select="count(current-group())"/></b></font>):<br/>
											<xsl:for-each select="current-group()"><a href="{saxon:path()}"><code><xsl:value-of select="node-name()"/></code> #<xsl:value-of select="position()"/></a><xsl:if test="position()&lt;last()">, </xsl:if>
											</xsl:for-each>
										</li>
									</xsl:for-each-group>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
				</ul>
				<hr/>
				
				<a name="Constants"><b><big>Constants:</big></b></a>
				<ul>
					<li>Undefined Constants<!--  (<code>constant-ref</code> without <code>constant-def</code>) -->
						<xsl:variable name="undefined-constants" select="//constant-ref[not(@name=//constant-def/@name)]"/>
						<xsl:choose>
							<xsl:when test="$undefined-constants"> (<font color="red"><b><xsl:value-of select="count($undefined-constants)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$undefined-constants" mode="undefined">
									<!-- <xsl:sort select="@name"/> -->
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Undefined Qualitative Constants<!--  (<code>qualitative-constant</code> without matching <code>qualitative-entry</code>) -->
						<xsl:variable name="undefined-qualitative-constants" select="//qualitative-constant[not(@value=(//(qualitative-entry/@entry|secondary-qualitative-entry/@name),'true','false'))]"/>
						<xsl:choose>
							<xsl:when test="$undefined-qualitative-constants"> (<font color="red"><b><xsl:value-of select="count($undefined-qualitative-constants)"/></b></font>):
								<ul>							
								<xsl:apply-templates select="$undefined-qualitative-constants" mode="undefined">
									<!-- <xsl:sort select="@value"/> -->
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Unused Constants<!--  (<code>constant-def</code> without any <code>constant-ref</code>'s) -->
						<xsl:variable name="unused-constants" select="//constant-def[not(@name=//constant-ref/@name)]"/>
						<xsl:choose>
							<xsl:when test="$unused-constants"> (<font color="red"><b><xsl:value-of select="count($unused-constants)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$unused-constants" mode="unused">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Duplicate Constants
						<xsl:variable name="duplicate-constants" select="for $v in //constant-def return if (count($v/ancestor::*/constant-def[@name=$v/@name])&gt;1) then $v else ()"/>
						<xsl:choose>
							<xsl:when test="$duplicate-constants"> (<font color="red"><b><xsl:value-of select="count(distinct-values($duplicate-constants/@name))"/></b></font>):
								<ul>
									<xsl:for-each-group select="$duplicate-constants" group-by="@name">
										<xsl:sort select="@name"/>
										<li>
											<xsl:variable name="local" select="current()/ancestor::value-defs"/>
											<xsl:choose>
												<xsl:when test="$local">local </xsl:when>
												<xsl:otherwise>global </xsl:otherwise>
											</xsl:choose>Constant "<i><xsl:value-of select="@name"/></i>"
											<xsl:if test="$local"> defined in Plan <a><xsl:attribute name="href"><xsl:apply-templates select="ancestor::plan" mode="xpath"/></xsl:attribute>"<i><xsl:value-of select="ancestor::plan/@name"/></i>"</a></xsl:if>
											(<font color="red"><b><xsl:value-of select="count(current-group())"/></b></font>):<br/>
											<xsl:for-each select="current-group()"><a href="{saxon:path()}"><code><xsl:value-of select="node-name()"/></code> #<xsl:value-of select="position()"/></a><xsl:if test="position()&lt;last()">, </xsl:if>
											</xsl:for-each>
										</li>
									</xsl:for-each-group>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
				</ul>
				<hr/> 
				
				<a name="Types"><b><big>Types:</big></b></a>
				<ul>
					<li>Undefined Types
						<xsl:variable name="undefined-types" select="(//parameter-def | //scalar-def | //constant-def | //argument)[not(lower-case(@type)=(/plan-library/domain-defs/domain/qualitative-scale-def/lower-case(@name), 'string', 'boolean', 'length', 'area', 'volume', 'mass', 'time', 'date', 'temperature', 'amount', 'energy', 'pressure', 'parts', 'currency', 'iterator'))]"/>
						<xsl:choose>
							<xsl:when test="$undefined-types"> (<font color="red"><b><xsl:value-of select="count($undefined-types)"/></b></font>):
								<ul>
								<xsl:for-each select="$undefined-types">
									<!-- <xsl:sort select="@type"/> -->
									<li>type "<i><xsl:value-of select="@type"/></i>" used in <a href="{saxon:path()}"><xsl:value-of select="node-name()"/></a> inside <xsl:call-template name="info"/></li>
								</xsl:for-each>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Unused Qualitative Types<!--  (<code>qualitative-scale-def</code>) -->
						<xsl:variable name="unused-types" select="//qualitative-scale-def[not(@name=(//parameter-def | //scalar-def | //constant-def | //argument)/@type)]"/>
						<xsl:choose>
							<xsl:when test="$unused-types"> (<font color="red"><b><xsl:value-of select="count($unused-types)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$unused-types" mode="unused">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Duplicate Qualitative Types
						<xsl:variable name="duplicate-types" select="for $d in //qualitative-scale-def return if (count(//qualitative-scale-def[@name=$d/@name])&gt;1) then $d else ()"/>
						<xsl:choose>
							<xsl:when test="$duplicate-types"> (<font color="red"><b><xsl:value-of select="count(distinct-values($duplicate-types/@name))"/></b></font>):
								<ul>
									<xsl:for-each-group select="$duplicate-types" group-by="@name">
										<xsl:sort select="@name"/>
										<li>Qualitative Type "<i><xsl:value-of select="@name"/></i>" (<font color="red"><b><xsl:value-of select="count(current-group())"/></b></font>):<br/>
											<xsl:for-each select="current-group()"><a href="{saxon:path()}"><code><xsl:value-of select="node-name()"/></code> #<xsl:value-of select="position()"/></a><xsl:if test="position()&lt;last()">, </xsl:if>
											</xsl:for-each>
										</li>
									</xsl:for-each-group>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>
						Wrong Qualitative-Types - <code>parameter-def</code> with <code>qualitative-parameter-def</code> inside, but without a type referring to a <code>qualitative-scale-def</code>
						<xsl:variable name="wrong-qualitative-types" select="//parameter-def[child::qualitative-parameter-def][not(@type=//qualitative-scale-def/@name)]"/>
						<xsl:choose>
							<xsl:when test="$wrong-qualitative-types"> (<font color="red"><b><xsl:value-of select="count($wrong-qualitative-types)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$wrong-qualitative-types" mode="unused">
									<!-- <xsl:sort select="@name"/> -->
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Type Mismatch<!--  (check for incompatible operands of comparison, comparison-def, value-description, value-range, variable-assignment (variable-ref, numerical-constant, qualitative-constant, constant-ref, argument-ref, operation, parameter-ref)) -->
						<ul>
						<xsl:for-each select="//comparison|//value-description|//variable-assignment|//return-value-assignment">
							<!-- |//comparison-def|//value-range -->
							<xsl:variable name="type1">
								<xsl:choose>
									<xsl:when test="self::value-description"><xsl:value-of select="//parameter-def[@name=current()/parent::parameter-proposition/@parameter-name]/@type"/></xsl:when>
									<xsl:when test="self::comparison"><xsl:apply-templates select="left-hand-side/*" mode="type"/></xsl:when>
									<!-- <xsl:when test="self::comparison-def"><xsl:apply-templates select="left-hand-parameter/*" mode="type"/></xsl:when> -->
									<xsl:when test="self::variable-assignment"><xsl:value-of select="(ancestor::plan/value-defs | /plan-library/domain-defs/domain | /plan-library/library-defs)/variable-def[@name=current/@variable]/scalar-def/@type"/></xsl:when>
									<xsl:when test="self::return-value-assignment"><xsl:value-of select="(ancestor::plan/value-defs | /plan-library/domain-defs/domain | /plan-library/library-defs)/variable-def[@name=current()/@variable-name]/scalar-def/@type"/></xsl:when>
									<xsl:otherwise>unknown</xsl:otherwise>
								</xsl:choose>
							</xsl:variable>
							<xsl:variable name="type2">
								<xsl:choose>
									<xsl:when test="self::value-description"><xsl:apply-templates select="if (tolerance) then *[last()-1] else *[last()]" mode="type"/></xsl:when>
									<xsl:when test="self::comparison"><xsl:apply-templates select="right-hand-side/*" mode="type"/></xsl:when>
									<!--	<xsl:when test="self::comparison-def"><xsl:apply-templates select="right-hand-parameter/*" mode="type"/></xsl:when> -->
									<xsl:when test="self::variable-assignment"><xsl:apply-templates select="*[last()]" mode="type"/></xsl:when>
									<xsl:when test="self::return-value-assignment"><xsl:apply-templates select="//plan[@name=current()/parent::plan-schema/@name]/returns/return-value[@name=current()/@return-value-name]/*" mode="type"/></xsl:when>
									<xsl:otherwise>unknown</xsl:otherwise>
								</xsl:choose>
							</xsl:variable>
							<xsl:if test="($type1!='') and ($type2!='') and ($type1!=$type2) and not(contains($type1,$type2)) and not(contains($type2,$type1))">
								<li>"<i><xsl:value-of select="$type1"/></i>" != "<i><xsl:value-of select="$type2"/></i>" at 
									<a href="{saxon:path()}"><code><xsl:value-of select="node-name()"/></code></a>, child of <a><xsl:attribute name="href"><xsl:apply-templates select=".." mode="xpath"/></xsl:attribute><code><xsl:value-of select="node-name(..)"/></code></a>, inside
									<xsl:call-template name="info"/>
								</li>
							</xsl:if> 					
						</xsl:for-each>
						</ul>
					</li>
				</ul>
				<hr/> 
				
				<a name="Units"><b><big>Units:</big></b></a>
				<ul>
					<li>Undefined Units
						<xsl:variable name="units" select="(//unit-def/@name, 'amount', $units-length, $units-temperature, $units-area, $units-volume, $units-mass, $units-time, $units-date, $units-energy, $units-pressure, $units-currency, $units-parts)"></xsl:variable>
						<xsl:variable name="undefined-units" select="(//limits|//numerical-constant | //raw-data-def)[@unit][not(@unit=$units)]|//(scalar-def|unit-def)[@default-unit][not(@default-unit=$units)]|//(scalar-def|unit-def)[@precision-unit][not(@precision-unit=$units)]"/>
						<xsl:choose>
							<xsl:when test="$undefined-units"> (<font color="red"><b><xsl:value-of select="count($undefined-units)"/></b></font>):
								<ul>
								<xsl:for-each select="$undefined-units">
									<!-- <xsl:sort select="@unit|@default-unit|@precision-unit"/> -->
									<li>unit "<i><xsl:value-of select="@unit|@default-unit|@precision-unit"/></i>" used in <a href="{saxon:path()}"><xsl:value-of select="node-name()"/></a> inside <xsl:call-template name="info"/></li>
								</xsl:for-each>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Unused Units<!--  (<code>unit-def</code>) -->
						<xsl:variable name="unused-units" select="//unit-def[not(@name=(//limits | //numerical-constant | //raw-data-def)/@unit|//(scalar-def|unit-def)/@default-unit|//(scalar-def|unit-def)/@precision-unit)]"/>
						<xsl:choose>
							<xsl:when test="$unused-units"> (<font color="red"><b><xsl:value-of select="count($unused-units)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$unused-units" mode="unused">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Duplicate Units
						<xsl:variable name="duplicate-units" select="for $d in //unit-def return if (count(//unit-def[@name=$d/@name])&gt;1) then $d else ()"/>
						<xsl:choose>
							<xsl:when test="$duplicate-units"> (<font color="red"><b><xsl:value-of select="count(distinct-values($duplicate-units/@name))"/></b></font>):
								<ul>
									<xsl:for-each-group select="$duplicate-units" group-by="@name">
										<xsl:sort select="@name"/>
										<li>Unit "<i><xsl:value-of select="@name"/></i>" (<font color="red"><b><xsl:value-of select="count(current-group())"/></b></font>):<br/>
											<xsl:for-each select="current-group()"><a href="{saxon:path()}"><code><xsl:value-of select="node-name()"/></code> #<xsl:value-of select="position()"/></a><xsl:if test="position()&lt;last()">, </xsl:if>
											</xsl:for-each>
										</li>
									</xsl:for-each-group>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
				</ul>
				<hr/>
				
				<a name="Contexts"><b><big>Contexts:</big></b></a>
				<ul>
					<li>Undefined Contexts<!--  (<code>context-ref</code> / <code>one-of</code> without <code>context-def</code> / <code>parameter-def (use-as-context="yes")</code>) -->
						<xsl:variable name="undefined-contexts" select="//set-context[not(@name=//context-def/@name)]|(//context-ref|//one-of)[not(@name=(//context-def | //parameter-def[*/@use-as-context='yes'])/@name)]"/>
						<xsl:choose>
							<xsl:when test="$undefined-contexts"> (<font color="red"><b><xsl:value-of select="count($undefined-contexts)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$undefined-contexts" mode="undefined">
									<!-- <xsl:sort select="@name"/> -->
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Unused Contexts<!--  (<code>context-def</code> without any <code>context-ref</code>'s) -->
						<xsl:variable name="unused-contexts" select="//context-def[not(@name=(//context-ref|//one-of)/@name)]"/>
						<xsl:choose>
							<xsl:when test="$unused-contexts"> (<font color="red"><b><xsl:value-of select="count($unused-contexts)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$unused-contexts" mode="unused">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Duplicate Contexts
						<xsl:variable name="duplicate-contexts" select="for $d in //context-def return if (count(//context-def[@name=$d/@name])&gt;1) then $d else ()"/>
						<xsl:choose>
							<xsl:when test="$duplicate-contexts"> (<font color="red"><b><xsl:value-of select="count(distinct-values($duplicate-contexts/@name))"/></b></font>):
								<ul>
									<xsl:for-each-group select="$duplicate-contexts" group-by="@name">
										<xsl:sort select="@name"/>
										<li>Context "<i><xsl:value-of select="@name"/></i>" (<font color="red"><b><xsl:value-of select="count(current-group())"/></b></font>):<br/>
											<xsl:for-each select="current-group()"><a href="{saxon:path()}"><code><xsl:value-of select="node-name()"/></code> #<xsl:value-of select="position()"/></a><xsl:if test="position()&lt;last()">, </xsl:if>
											</xsl:for-each>
										</li>
									</xsl:for-each-group>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
				</ul>
				<hr/>

				<a name="Misc"><b><big>Misc. Stuff:</big></b></a>
				<ul>
					<li>List of all <i>and/or/xor</i> with only one child
						<xsl:variable name="logcomb" select="(//simple-condition-combination|//constraint-combination|//context-combination)[count(*)&lt;2]"/>
						<xsl:choose>
							<xsl:when test="$logcomb"> (<font color="red"><b><xsl:value-of select="count($logcomb)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$logcomb" mode="undefined">
									<!-- <xsl:sort select="@name"/> -->
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li><code>numerical-constant</code> without <code>unit</code> attribute
						<xsl:variable name="numconst" select="//numerical-constant[not(@unit)]"/>
						<xsl:choose>
							<xsl:when test="$numconst"> (<font color="red"><b><xsl:value-of select="count($numconst)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$numconst" mode="undefined">
									<!-- <xsl:sort select="@name"/> -->
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>List of all <code>to-be-defined</code> and <code>not-yet-defined</code>
						<xsl:variable name="todos" select="//to-be-defined|//not-yet-defined"/>
						<xsl:choose>
							<xsl:when test="$todos"> (<font color="red"><b><xsl:value-of select="count($todos)"/></b></font>):
								<ul>
									<xsl:apply-templates select="$todos" mode="undefined"/>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Wrong <code>refer-to</code> (undefined label or missing <code>refers-to</code>)
						<xsl:variable name="refer-tos" select="for $r in /plan-library/plans//refer-to return if (not(//plan[@name= $r/ancestor::plan/refers-to[(@plan-name|@as)=$r/@plan-name]/@plan-name]//*[@label=$r/@label])) then $r else ()"/>
						<xsl:choose>
							<xsl:when test="$refer-tos"> (<font color="red"><b><xsl:value-of select="count($refer-tos)"/></b></font>):
								<ul>
									<xsl:apply-templates select="$refer-tos" mode="undefined"/>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>time-annotation with time-range with only duration but with 'reference' unequal 'now'
						<xsl:variable name="time-annotation" select="//time-annotation[time-range/duration][not(time-range/(starting-shift|finishing-shift))][not(./now)]"/>
						<xsl:choose>
							<xsl:when test="$time-annotation"> (<font color="red"><b><xsl:value-of select="count($time-annotation)"/></b></font>):
								<ul>
									<xsl:apply-templates select="$time-annotation" mode="undefined"/>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li>cyclical-plan without repeat-specification
						<xsl:variable name="cyclical-plan" select="//cyclical-plan[repeat-specification[not(*)]]"/>
						<xsl:choose>
							<xsl:when test="$cyclical-plan"> (<font color="red"><b><xsl:value-of select="count($cyclical-plan)"/></b></font>):
								<ul>
								<xsl:apply-templates select="$cyclical-plan" mode="undefined">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
						</xsl:choose>
					</li>
					<li><i>complex-time-def</i>:<ul>
						<li><code>time-annotation-ref</code> without matching <code>time-annotation-assignment</code>
							<xsl:variable name="undefined-time-annotation-assignment" select="//time-annotation-ref[not(@name=//time-annotation-assignment/@name)]"/>
							<xsl:choose>
								<xsl:when test="$undefined-time-annotation-assignment"> (<font color="red"><b><xsl:value-of select="count($undefined-time-annotation-assignment)"/></b></font>):
									<ul>
									<xsl:apply-templates select="$undefined-time-annotation-assignment" mode="undefined">
										<!-- <xsl:sort select="@name"/> -->
									</xsl:apply-templates>
									</ul>
								</xsl:when>
								<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
							</xsl:choose>		
						</li>
						<li><code>cyclical-time-point-ref</code> without matching <code>cyclical-time-point-assignment</code>
							<xsl:variable name="undefined-cyclical-time-point-assignment" select="//cyclical-time-point-ref[not(@name=//cyclical-time-point-assignment/@name)]"/>
							<xsl:choose>
								<xsl:when test="$undefined-cyclical-time-point-assignment"> (<font color="red"><b><xsl:value-of select="count($undefined-cyclical-time-point-assignment)"/></b></font>):
									<ul>
									<xsl:apply-templates select="$undefined-cyclical-time-point-assignment" mode="undefined">
										<!-- <xsl:sort select="@name"/> -->
									</xsl:apply-templates>
									</ul>
								</xsl:when>
								<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
							</xsl:choose>		
						</li>
						<li><code>cyclical-time-annotation-ref</code> without matching <code>cyclical-time-annotation-assignment</code>
							<xsl:variable name="undefined-cyclical-time-annotation-assignment" select="//cyclical-time-annotation-ref[not(@name=//cyclical-time-annotation-assignment/@name)]"/>
							<xsl:choose>
								<xsl:when test="$undefined-cyclical-time-annotation-assignment"> (<font color="red"><b><xsl:value-of select="count($undefined-cyclical-time-annotation-assignment)"/></b></font>):
									<ul>
									<xsl:apply-templates select="$undefined-cyclical-time-annotation-assignment" mode="undefined">
										<!-- <xsl:sort select="@name"/> -->
									</xsl:apply-templates>
									</ul>
								</xsl:when>
								<xsl:otherwise>: <font color="green"><b>none</b></font></xsl:otherwise>
							</xsl:choose>		
						</li>
					</ul></li>
				</ul>
			</body>
		</html>
	</xsl:template>

	<xsl:template match="*" mode="unused">
		<li><a href="{saxon:path()}"><xsl:value-of select="@name"/></a><!--  (line #<xsl:value-of select="saxon:line-number(.)"/>) --></li>
	</xsl:template>

	<xsl:template match="*" mode="undefined">
		<li>
			<xsl:choose>
				<xsl:when test="@name|@variable-name|@plan-name|@parameter-name|@parameter|@value[parent::qualitative-constant]|@unit|@default-unit|@precision-unit">
					<code><xsl:value-of select="node-name()"/></code>&#160;<nobr><a href="{saxon:path()}">"<i><xsl:value-of select="@name|@variable-name|@plan-name|@parameter-name|@parameter|@value[parent::qualitative-constant]|@unit|@default-unit|@precision-unit"/></i>"</a><!--  (line #<xsl:value-of select="saxon:line-number()"/>) -->,</nobr> inside
				</xsl:when>
				<xsl:otherwise>
					<a href="{saxon:path()}"><code><xsl:value-of select="node-name()"/></code></a><!--  (line #<xsl:value-of select="saxon:line-number()"/>) -->, inside
				</xsl:otherwise>
			</xsl:choose>
		<xsl:call-template name="info"/>
		</li>
	</xsl:template>
	
	<xsl:template name="info">
		<xsl:choose>
			<xsl:when test="ancestor::parameter-def">
				<code>parameter-def </code><nobr><a><xsl:attribute name="href"><xsl:apply-templates select="ancestor::parameter-def" mode="xpath"/></xsl:attribute>"<xsl:value-of select="ancestor::parameter-def/@name"/>"</a></nobr>
			</xsl:when>
			<xsl:when test="not(self::parameter-def) and ancestor::domain">				
				<i><a><xsl:attribute name="href"><xsl:apply-templates select="ancestor::domain" mode="xpath"/></xsl:attribute>domain</a></i>
			</xsl:when>
			<xsl:when test="ancestor::plan">
				<xsl:choose>
					<xsl:when test="ancestor::conditions">
						<i>condition </i><nobr><a><xsl:attribute name="href"><xsl:apply-templates select="ancestor::conditions/*[1]" mode="xpath"/></xsl:attribute><code><xsl:value-of select="node-name(ancestor::conditions/*[1])"/></code></a><xsl:text>, in </xsl:text></nobr>
					</xsl:when>
					<xsl:when test="ancestor::intentions">
						<nobr><code><a><xsl:attribute name="href"><xsl:apply-templates select="ancestor::intention" mode="xpath"/></xsl:attribute>intention</a></code><xsl:text>, in </xsl:text></nobr>
					</xsl:when>
				</xsl:choose>
				<code>plan </code><nobr><a><xsl:attribute name="href"><xsl:apply-templates select="ancestor::plan" mode="xpath"/></xsl:attribute>"<i><xsl:value-of select="ancestor::plan/@name"/></i>"</a></nobr>
			</xsl:when>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="*" mode="xpath">
		<xsl:value-of select="saxon:path()"/>
	</xsl:template>
	
	<xsl:template match="qualitative-constant" mode="type"><xsl:value-of select="//qualitative-scale-def[qualitative-entry/@entry=current()/@value]/@name"/></xsl:template>
	<xsl:template match="numerical-constant" mode="type">
		<xsl:value-of select="if (@unit=($units-time,$units-date)) then 'date/time' else
			if (@unit=$units-length) then 'length' else 
			if (@unit=$units-temperature) then 'temperature' else 
			if (@unit=$units-area) then 'area' else 
			if (@unit=$units-volume) then 'volume' else 
			if (@unit=$units-mass) then 'mass' else 		 
			if (@unit=$units-energy) then 'energy' else 
			if (@unit=$units-pressure) then 'pressure' else 
			if (@unit=$units-currency) then 'currency' else 
			if (@unit=$units-parts) then 'parts' else 
			if (contains(@value,'.')) then 'unknown' else 'amount'"/>
	</xsl:template>
	<xsl:template match="string-constant" mode="type">string</xsl:template>
	<xsl:template match="operation" mode="type"><xsl:apply-templates select="*[1]" mode="type"/></xsl:template>	
	<xsl:template match="constant-ref" mode="type"><xsl:value-of select="(ancestor::plan/value-defs | /plan-library/domain-defs/domain | /plan-library/library-defs)/constant-def[@name=current()/@name]/@type"/></xsl:template>
	<xsl:template match="variable-ref" mode="type"><xsl:value-of select="(ancestor::plan/value-defs | /plan-library/domain-defs/domain | /plan-library/library-defs)/variable-def[@name=current()/@name]/scalar-def/@type"/></xsl:template>	
	<xsl:template match="parameter-ref" mode="type"><xsl:value-of select="//parameter-def[@name=current()/@name]/@type"/></xsl:template>
	<xsl:template match="argument-ref" mode="type"><xsl:value-of select="ancestor::plan//argument[@name=current()/@name]/@type"/></xsl:template>
</xsl:stylesheet>
