<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:saxon="http://saxon.sf.net/" xmlns:file="java:java.io.File">
	<xsl:output method="html" indent="yes"/>
	
	<!-- match root node -->
	<xsl:template match="/">
		<xsl:variable name="file-name" select="file:getName(file:new(string(base-uri())))"/>
		<html>
			<head>
				<title>Consistency Check of GMT-Macros file <xsl:value-of select="$file-name"/></title>
				<style type="text/css">
					<xsl:comment>
						a { text-decoration:none; font-weight:normal; white-space:nowrap; }
						ul { margin-top:-3px; margin-bottom:-3px  }
					</xsl:comment>
				</style>
			</head>
			<body>
				<h2>Consistency Check of GMT-Macros file <i><xsl:value-of select="$file-name"/></i></h2>
				<b><big>Macros: </big></b><ul>
					<li><font color="maroon"><b>warning</b></font> - <code>macro</code> without any matching <code>macro-ref</code>'s:
						<!-- all uncalled macro's -->
						<xsl:variable name="uncalled-macros" select="//macro[not(@name=//macro-ref/@name)]"/>
						<xsl:choose>
							<xsl:when test="$uncalled-macros">(<b><xsl:value-of select="count($uncalled-macros)"/></b>)
								<ul>
								<xsl:apply-templates select="$uncalled-macros" mode="unused">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>none</xsl:otherwise>
						</xsl:choose>
					</li>
					<li><font color="red"><b>severe</b></font> - <code>macro-ref</code> and <code>macro-include</code> elements without corresponding <code>macro</code>:
						<!-- all macro-ref's in this file that refers to no macro -->
						<xsl:variable name="undefined-macros" select="(//macro-ref|//macro-include)[not(@name=//macro/@name)]"/>
						<xsl:choose>						
							<xsl:when test="$undefined-macros">(<b><xsl:value-of select="count($undefined-macros)"/></b>)
								<ul>
								<xsl:apply-templates select="$undefined-macros" mode="undefined">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>none</xsl:otherwise>
						</xsl:choose>
					</li>
					<li><font color="teal"><b>hint</b></font> - Macros without Links:
						<xsl:variable name="linkless-macros" select="//macro[not(.//(GMT-LINK|GMT-LINK_EXT))]"/>
						<xsl:choose>						
							<xsl:when test="$linkless-macros">(<b><xsl:value-of select="count($linkless-macros)"/></b>)
								<ul>
								<xsl:apply-templates select="$linkless-macros" mode="unused">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>none</xsl:otherwise>
						</xsl:choose>
					</li>
					<li><font color="teal"><b>hint</b></font> - empty Macros:
						<xsl:variable name="empty-macros" select="//macro[not(*)]"/>
						<xsl:choose>						
							<xsl:when test="$empty-macros">(<b><xsl:value-of select="count($empty-macros)"/></b>)
								<ul>
								<xsl:apply-templates select="$empty-macros" mode="unused">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>none</xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Statistics:<ul>
						<li><xsl:value-of select="count(//macro)"/> macro(s)</li>
						<li><xsl:value-of select="count(//macro[.//(GMT-LINK|GMT-LINK_EXT)])"/> macro(s) with links</li>
						<li><xsl:value-of select="format-number(avg(//macro/count(.//*)),'0.0')"/> element(s) per macro (average)</li>
					</ul></li>
				</ul>
				<hr/>
				<b><big>Groups: </big></b><ul>
					<li><font color="red"><b>severe</b></font> - <code>group-link</code> and <code>group-include</code> without <code>group</code>:
						<xsl:variable name="undefined-groups" select="(//group-link|//group-include)[not(@name=//group/@name)]"/>
						<xsl:choose>						
							<xsl:when test="$undefined-groups">(<b><xsl:value-of select="count($undefined-groups)"/></b>)
								<ul>
								<xsl:apply-templates select="$undefined-groups" mode="undefined">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>none</xsl:otherwise>
						</xsl:choose>
					</li>
					<li><font color="teal"><b>hint</b></font> - empty Groups:
						<xsl:variable name="empty-groups" select="//group[not(*)]"/>
						<xsl:choose>						
							<xsl:when test="$empty-groups">(<b><xsl:value-of select="count($empty-groups)"/></b>)
								<ul>
								<xsl:apply-templates select="$empty-groups" mode="unused">
									<xsl:sort select="@name"/>
								</xsl:apply-templates>
								</ul>
							</xsl:when>
							<xsl:otherwise>none</xsl:otherwise>
						</xsl:choose>
					</li>
					<li>Statistics:<ul>
						<li><xsl:value-of select="count(//group)"/> group(s)</li>
						<li><xsl:value-of select="count(//macro-ref)"/> macro-ref(s)</li>
						<li><xsl:value-of select="format-number(avg(//group/count(*)),'0.0')"/> element(s) per group (average)</li>
					</ul></li>
				</ul>
			</body>
		</html>
	</xsl:template>
	
	<!-- template for unused paramter-def's -->
	<xsl:template match="*" mode="unused">
		<li><a href="{saxon:path()}"><xsl:value-of select="@name"/></a></li>
	</xsl:template>
	
	<!-- template for parameter-ref's without matching paramter-def -->
	<xsl:template match="*" mode="undefined">
		<li><a href="{saxon:path()}"><xsl:value-of select="@name"/></a> 
		<br/>&#160;&#160;<xsl:value-of select="node-name(.)"/>, child of <i><xsl:value-of select="node-name(..)"/></i>
		</li>
	</xsl:template>
</xsl:stylesheet>
