<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:saxon="http://saxon.sf.net/" xmlns:file="java:java.io.File">
	<xsl:output method="html" indent="yes"/>
	<xsl:strip-space elements="*"/>
	
	<xsl:param name="group_color">navy</xsl:param>
	<xsl:param name="group_link_color">green</xsl:param>
	<xsl:param name="macro_color">olive</xsl:param>
	<xsl:param name="label_color">#505050</xsl:param>
	<xsl:param name="description_color">gray</xsl:param>
	
	<xsl:param name="element_color">#900090</xsl:param>
	<xsl:param name="attr_name_color">#900000</xsl:param>
	<xsl:param name="attr_value_color">#800080</xsl:param>
	<xsl:param name="text_color">#000000</xsl:param>
	<xsl:param name="comment_color">#808080</xsl:param>
	<xsl:param name="pi_color">#008080</xsl:param>
	<xsl:param name="symbol_color">#0000FF</xsl:param>
	
	<xsl:template match="/">
		<xsl:variable name="file-name" select="file:getName(file:new(string(base-uri())))"/>
		<html>
			<head>
				<title>HTML documentation of <xsl:value-of select="$file-name"/></title>
				<style type="text/css">
					<xsl:comment>
						a { text-decoration:none; font-weight:normal; white-space:nowrap; }
	ul {list-style-type:disc }
	ul ul {list-style-type:circle }
	ul ul ul {list-style-type:square }
	ul ul ul ul {list-style-type:disc }
	ul ul ul ul ul {list-style-type:circle }
	ul ul ul ul ul ul {list-style-type:square }					
	ul ul ul ul ul ul ul {list-style-type:disc }
	ul ul ul ul ul ul ul ul {list-style-type:circle }
	ul ul ul ul ul ul ul ul ul {list-style-type:square }					
					
	.group_name { color:<xsl:value-of select="$group_color"/>;font-style:italic }
	.macro_name { color:<xsl:value-of select="$macro_color"/>;font-style:italic }
	.element_name { color:<xsl:value-of select="$element_color"/>;font-style:italic }	
	.label_name { color:<xsl:value-of select="$label_color"/>;font-style:italic }	
	.description { color:<xsl:value-of select="$description_color"/>;font-style:italic }	
	.group_link_label { color:<xsl:value-of select="$label_color"/>; font-size:small }
	
	 a.group_link { color:<xsl:value-of select="$group_link_color"/>; text-decoration:none; font-style:italic }
	 a.macro_ref {color:<xsl:value-of select="$macro_color"/>; text-decoration:none; font-style:italic }
	 
	dl { margin-left:20px; margin-top:-3px; margin-bottom:-3px  }
	dl.root { margin-left:5px; margin-top:-3px; margin-bottom:-3px  }
	.element { color:<xsl:value-of select="$element_color"/> }
	.attr_name { color:<xsl:value-of select="$attr_name_color"/> }
	.attr_value { color:<xsl:value-of select="$attr_value_color"/>; font-style:italic }
	.text { color:<xsl:value-of select="$text_color"/>" }
	.comment { color:<xsl:value-of select="$comment_color"/>" }
	.pi { color:<xsl:value-of select="$pi_color"/> }
	.symbol { color:<xsl:value-of select="$symbol_color"/> }		
</xsl:comment>
				</style>
			</head>
			<body>
				<h2>automatically generated HTML documentation of <i><xsl:value-of select="$file-name"/></i></h2>
				<ul>
					<li><a href="#root_group">Groups</a></li>
					<li><a href="#macros">Macros</a></li>
				</ul>
				<hr/>
				<h3><a name="root_group">Groups:</a>	</h3>
				<xsl:apply-templates select="/gmt-macros/structure-def"/>
				<hr/>
				<h3><a name="macros">Macros:</a></h3>
				<xsl:apply-templates select="/gmt-macros/macros-def"/>
			</body>
		</html>
	</xsl:template>
	<xsl:template match="/gmt-macros/structure-def">
		<ul>
			<xsl:apply-templates/>
		</ul>
	</xsl:template>
	<xsl:template match="/gmt-macros/macros-def">
		<ol>
			<xsl:apply-templates select="descendant-or-self::macro">
				<xsl:sort select="@name"/>
			</xsl:apply-templates>
		</ol>
	</xsl:template>	
	<xsl:template match="macro">
		<li>
			<a href="{saxon:path()}" name="m_{@name}"><b>Macro:</b></a>&#160;
			<span class="macro_name"><xsl:value-of select="@name"/></span>
			
			<!-- <xsl:if test="self::node()[@inputRequired='true']">
				<small><xsl:text> - User-Input</xsl:text></small>
			</xsl:if> -->
			<xsl:if test=".//GMT-LINK|.//GMT-LINK_EXT">
				<small><xsl:text> (contains GMT-link)</xsl:text></small>
			</xsl:if>
			<xsl:variable name="macro_name" select="@name"/>
			<xsl:choose>
				<xsl:when test="/gmt-macros/structure-def//macro-ref[@name=$macro_name]">
					Used in group 
					<xsl:if test="/gmt-macros/structure-def/macro-ref[@name=$macro_name]">
						<a href="#root_group" class="group_link">root-group</a>
						<xsl:if test="/gmt-macros/structure-def//group[macro-ref[@name=$macro_name]]">,&#160;</xsl:if>
					</xsl:if>
						<xsl:for-each select="/gmt-macros/structure-def//group[macro-ref[@name=$macro_name]]">
							<a class="group_link">
								<xsl:attribute name="href">#g_<xsl:value-of select="translate(@name,' ','_')"/></xsl:attribute>
								<xsl:value-of select="@name"/>
							</a>
							<xsl:if test="position() != last()">,&#160;</xsl:if>
						</xsl:for-each>.
					</xsl:when>
				<xsl:otherwise>					
					This macro is <b>never</b> used!					
				</xsl:otherwise>
			</xsl:choose>
		</li>
	</xsl:template>
	<xsl:template match="//group">
		<li>
			<xsl:element name="a">
				<xsl:attribute name="href"><xsl:value-of select="saxon:path()"/></xsl:attribute>
				<xsl:attribute name="name">g_<xsl:value-of select="translate(@name,' ','_')"/></xsl:attribute>
				<b>Group:&#160;<span class="group_name"><xsl:value-of select="@name"/></span></b>
			</xsl:element>
			<xsl:if test="self::node()[@description]">
				<br/>&#160;&#160;&#160;&#160;
				<small>
					<b>Description:</b><span class="description"><xsl:value-of select="@description"/></span>
				</small>
			</xsl:if>
			<ul>
				<xsl:apply-templates/>
			</ul>
		</li>
	</xsl:template>
	<xsl:template match="/gmt-macros/structure-def//macro-ref">
		<li>
			<a href="{saxon:path()}"><b>Reference to Macro:</b></a>&#160;
			<a href="#m_{@name}" class="macro_ref"><xsl:value-of select="@name"/></a>
			<xsl:variable name="macro_name" select="@name"/>
			<xsl:if test="/gmt-macros/macros-def//macro[@name=$macro_name][@inputRequired='true']">
				<small><xsl:text> - User-Input</xsl:text></small>
			</xsl:if>
			<xsl:if test="/gmt-macros/macros-def//macro[@name=$macro_name][@hasLink='true']">
				<small><xsl:text> (contains GMT-link)</xsl:text></small>
			</xsl:if>
			<xsl:if test="/gmt-macros/macros-def//macro[@name=$macro_name][@description]">
				<br/>&#160;&#160;&#160;&#160;
				<small>
					<b>Description:</b>
					<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>
					<span class="description">
						<xsl:value-of select="/gmt-macros/macros-def//macro[@name=$macro_name]/@description"/>
					</span>
				</small>
			</xsl:if>
		</li>
	</xsl:template>
	<xsl:template match="//single-element-ref">
		<li>
			<a href="{saxon:path()}"><b>Element:</b></a>&#160;
			<span class="element_name"><xsl:value-of select="@name"/></span>
			<xsl:if test="self::node()[@description]">
				<br/>&#160;&#160;&#160;&#160;
				<small>
					<b>Description:</b>&#160;					
					<span class="description">
						<xsl:value-of select="@description"/>
					</span>
				</small>
			</xsl:if>
		</li>
	</xsl:template>
	<xsl:template match="/gmt-macros/structure-def//group-link">
		<li>
			<a href="{saxon:path()}"><b>Link to group:</b></a>&#160;
			<xsl:element name="a">
				<xsl:attribute name="href">#g_<xsl:value-of select="translate(@name,' ','_')"/></xsl:attribute>
				<xsl:attribute name="class">group_link</xsl:attribute>
				<xsl:value-of select="@name"/>
			</xsl:element>
			<xsl:if test="self::node()[@label]">  
				<span class="group_link_label">
					<xsl:text> (</xsl:text>
					<xsl:value-of select="@label"/>
					<xsl:text>)</xsl:text>
				</span>
			</xsl:if>
			<xsl:if test="self::node()[@description]">
				<br/>&#160;&#160;&#160;&#160;
				<small>
					<b>Description:</b>&#160;
					<span class="description">
						<xsl:value-of select="@description"/>
					</span>
				</small>
			</xsl:if>
		</li>
	</xsl:template>
	<xsl:template match="/gmt-macros/structure-def//label">
		<li>
			<b>Label:</b>&#160;
			<span class="label_name">
				<xsl:value-of select="@text"/>
			</span>
		</li>
	</xsl:template>
	<xsl:template match="/gmt-macros/structure-def//separator">
		<!-- <dt><hr/></dt> --><br/>
	</xsl:template>
</xsl:stylesheet>
